
#ifndef __CHB_COLORSPACE_CONVERT
#define __CHB_COLORSPACE_CONVERT

#include <math.h>

static void chb_rgb_to_xyz(float const * rgb, float * XYZ)
{
	// from https://en.wikipedia.org/wiki/CIE_1931_color_space
	// supposedly from Fairman et al. How the CIE 1931 color-matching functions were derived from Wright-Guild data
	// DOI: 10.1002/(SICI)1520-6378(199702)22:1<11::AID-COL4>3.0.CO;2-7
	float scale = 1.0f / 0.17697f;
	XYZ[0] = 0.49f    * rgb[0] + 0.31f    * rgb[1] + 0.20f    * rgb[2];
	XYZ[1] = 0.17697f * rgb[0] + 0.81240f * rgb[1] + 0.01063f * rgb[2];
	XYZ[2] = 0.00f    * rgb[0] + 0.01f    * rgb[1] + 0.99f    * rgb[2];

	for(unsigned int i=0; i<3; i++)
		XYZ[i] = XYZ[i] * scale;
}

static void chb_xyz_to_rgb(float const * XYZ, float * rgb)
{
	rgb[0] =  0.41847f    * XYZ[0] + -0.15866f   * XYZ[1] + -0.082835f * XYZ[2];
	rgb[1] = -0.091169f   * XYZ[0] +  0.25243f   * XYZ[1] +  0.015708f * XYZ[2];
	rgb[2] =  0.00092090f * XYZ[0] + -0.0025498f * XYZ[1] + -0.17860f  * XYZ[2];
}

static float chb_xyz_to_lab__f(float const t)
{
	if(t > powf( (6.0f/29.0f), 3))
		return powf(t, 1.0f/3.0f);

	return (1.0f/3.0f) * powf( (29.0f/6.0f), 2) * t + (4.0f/29.0f);
}

static void chb_xyz_to_lab(float const * XYZ, float * Lab)
{
	//http://en.wikipedia.org/wiki/Lab_color_space
	//float (f*)(float) = chb_xyz_to_lab__f;

	Lab[0] = 116 * chb_xyz_to_lab__f( XYZ[1] ) - 16;
	Lab[1] = 500 * (chb_xyz_to_lab__f( XYZ[0] ) - chb_xyz_to_lab__f( XYZ[1] ));
	Lab[1] = 200 * (chb_xyz_to_lab__f( XYZ[1] ) - chb_xyz_to_lab__f( XYZ[2] ));
}

static void chb_rgb_to_lab(float const * rgb, float * lab)
{
	float xyz[3];
	chb_rgb_to_xyz(rgb, xyz);
	chb_xyz_to_lab(xyz, lab);
}

static void chb_to_float_rgb(chb_rgb const * rgb_c, float * rgb_f)
{
	for(int i=0; i<3; i++)
		rgb_f[i] = chb_rgb_get_channel(rgb_c, i) / 255.0f;
}

#endif
