#ifndef __VIEW_H
#define __VIEW_H

#include <stdio.h>
#include <string.h>
#include "view_resize.h"
#include "view_term.h"

#include "chb.h"

unsigned int chb_depth;
unsigned int chb_char;
unsigned int arg_img=1;
unsigned int replace_frame = 0;

typedef void (*loader_func) (char const *, size_t *, size_t *, unsigned char **);

static void parse_depth(char const * arg)
{
	if(strncmp("3", arg, 1) == 0)
		chb_depth = CHB_DEPTH_ANSI_3BIT;
	else if(strncmp("4", arg, 1) == 0)
		chb_depth = CHB_DEPTH_ANSI_4BIT;
	else if(strncmp("8", arg, 1) == 0)
		chb_depth = CHB_DEPTH_ANSI_8BIT;
	else if(strncmp("24", arg, 1) == 0)
		chb_depth = CHB_DEPTH_ANSI_24BIT;
	else if(strncmp("ms", arg, 1) == 0)
		chb_depth = CHB_DEPTH_MS_CONSOLE;
	else
		chb_depth = CHB_DEPTH_BEST;
}

static void parse_char(char const * arg)
{
	if(strncmp("utf8", arg, 4) == 0)
		chb_char = CHB_CHAR_UTF8;
	else if(strncmp("utf16", arg, 5) == 0)
		chb_char = CHB_CHAR_UTF16;
	else if(strncmp("wide", arg, 4) == 0)
		chb_char = CHB_CHAR_WIDE;
	else if(strncmp("cp437", arg, 5) == 0)
		chb_char = CHB_CHAR_CP437;
	else
		chb_char = CHB_CHAR_BEST;
}

static void checkArgs(int const argc, char * * argv)
{
	int arg_i;
	for(arg_i=1; arg_i<argc; arg_i++)
	{
		char const * arg_s = argv[arg_i];
		if( strncmp(arg_s, "-d", 2) == 0)
		{
			arg_i++;
			parse_depth(argv[arg_i]);
		}
		else if( strncmp(arg_s, "-c", 2) == 0)
		{
			arg_i++;
			parse_char(argv[arg_i]);
		}
		else if( strncmp(arg_s, "-f", 2) == 0)
		{
			arg_i++;
			replace_frame = 1;
		}
		else
		{
			break;
		}
	}

	unsigned char more_args = arg_i < argc;

   if(argc < 2 || !more_args)
   {
      printf("Usage: %s [options] image_path\n", argv[0]);
		printf("Options:\n");
		printf("\t-d depth\t3, 4, 8, 24, ms\n");
		printf("\t-c char\tutf8, utf16, wide, cp437\n");
		printf("\t-f     \treplace image in place\n");
      exit(0);
   }

	arg_img = arg_i;
}

static void checkFile(char const * filepath)
{
	FILE * f;

	f = fopen(filepath, "rb");
	if(f == NULL)
	{
		printf("Unable to open %s\n", filepath);
		exit(2);
	}

	fclose(f);

	if(!replace_frame)
		printf("%s ", filepath);
}

static void displayImage(unsigned int image_w, unsigned int image_h, unsigned char const * rgb_image)
{
	unsigned int canvas_w = get_canvas_width();
	unsigned int canvas_h = get_canvas_height();

	unsigned int target_w = get_target_w(image_w, image_h, canvas_w, canvas_h);
	unsigned int target_h = get_target_h(image_w, image_h, canvas_w, canvas_h);

	if(!replace_frame) {
		printf(": console %dx%d ", canvas_w, canvas_h);
		printf(": output %dx%d ", target_w, target_h);
	}

	//printf("%zux%zu to %ux%u = %ux%u\n", image_w, image_h, canvas_w, canvas_h, target_w, target_h);

	char smaller_than_canvas = target_w == image_w;

	if(!replace_frame) {
		printf(": fit wo scale %i ", smaller_than_canvas);
		printf("\n");
	}

	chb_frame_state state;
	if(smaller_than_canvas)
	{
		chb_init_frame(image_w, image_h, &state);
		if(replace_frame)
			chb_print_frame_opt(&state, chb_depth, chb_char, 0, 0, image_w, image_h, rgb_image);
		else
			chb_print_image_opt(chb_depth, chb_char, image_w, image_h, rgb_image);
		chb_free_frame(&state);
	}
	else
	{
		chb_init_frame(target_w, target_h, &state);
		unsigned char * scaled_rgb = resize_rgb_image(image_w, image_h, rgb_image, target_w, target_h);
		if(replace_frame)
			chb_print_frame_opt(&state, chb_depth, chb_char, 0, 0, target_w, target_h, scaled_rgb);
		else
			chb_print_image_opt(chb_depth, chb_char, target_w, target_h, scaled_rgb);
		free(scaled_rgb);
		chb_free_frame(&state);
	}
}

static void showImages(int argc, char ** argv, loader_func loadImage)
{
	chb_depth = CHB_DEPTH_BEST;
	chb_char = CHB_CHAR_BEST;

	checkArgs(argc, argv);

	size_t width, height;
	unsigned char * rgb_image;

	for(int i=arg_img; i<argc; i++)
	{
		checkFile(argv[i]);
		loadImage(argv[i], &width, &height, &rgb_image);

		if(!replace_frame)
			printf(": input %zux%zu ", width, height);

		displayImage(width, height, rgb_image);
		free(rgb_image);
	}
}

#endif

