#include <stdlib.h>

#ifdef WIN32
#include <windows.h>
#include <wincodec.h>
#endif

#include "view_impl.h"


void getWICImage(char const * filepath, size_t * out_width, size_t * out_height, unsigned char * * rgb_image)
{
	CoInitialize();
	{
	size_t width, height;

	HRESULT status;
	CComPtr<IWICImagingFactory> wicFactory;
	CComPtr<IWICBitmapDecoder> wicDecoder;
	CComPtr<IWICBitmapFrameDecode> wicFrame;
	CComPtr<IWICFormatConverter> wicConverter;
	unsigned int frameCount = 0;

	status = CoCreateInstance(
            CLSID_WICImagingFactory,
            NULL,
            CLSCTX_INPROC_SERVER,
            IID_PPV_ARGS(&wicFactory)
	);
	if(FAILED(status)) {
		printf("Error making WIC COM object\n")
		exit(1);
	}

	wicFactory->CreateDEcoderFromFilename(
	    filepath, 
	    NULL, 
	    GENERIC_READ, 
	    WICDecodeMetadataCacheOnLoad, 
	    &wicDecoder
	);
	if(FAILED(status)) {
		printf("Error opening %s\n", filepath);
		exit(1);
	}

	status = wicDecoder->GetFrameCount(&frameCount);
	if(FAILED(status)) {
		printf("Error reading %s\n", filepath);
		exit(1);
	}

	unsigned int firstFrame = 0;
	status = wicDecoder->GetFrame(firstFrame, &wicFrame);
	if(FAILED(status)) {
		printf("Error reading image data\n");
		exit(1);
	}

	status = wicFrame->GetSize(&width, &height);
	if(FAILED(status)) {
		printf("Error reading image dimensions\n");
		exit(1);
	}

	status = wicFactory->CreateFormatConverter(&wicConverter);
	if(FAILED(status)) {
		printf("Error making WIC convertor\n");
		exit(1);
	}

	status = wicConverter->Initialize(
	    wicFrame,
	    GUID_WICPixelFormat24bppPBGRA, 
	    WICBitmapDitherTypeNone, 
	    NULL,
	    0.f, 
	    WICBitmapPaletteTypeCustom
	);
	if(FAILED(status)) {
		printf("Error initializing WIC converter\n");
		exit(1);
	}

	size_t size = height * width * 3 * sizeof(unsigned char);
	size_t stride = width * 3;
	(*rgb_image) = malloc(size);

	wicConverter->CopyPixels(NULL, stride, size, *rgb_image);

	}
	CoUninitialize():

}

int main(int argc, char ** argv)
{
	showImages(argc, argv, getWICImage);
	return 0;
}

